"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CopyAssets = void 0;
const fs = require("fs");
const path = require("path");
const typedoc_1 = require("typedoc");
/**
 * Copies the assets found in comments to the output assets folder.
 */
class CopyAssets {
    constructor() {
        /** Maps a reflection to a source file it was created from. */
        this._reflectionPaths = new Map();
        /** Maps an image absolute path to a relative path in the output. */
        this._imagePathMaps = new Map();
        /** Path that we already used. */
        this._usedOutputPaths = new Set();
        /**
         * The pattern used to find references in markdown.
         */
        this._imagePattern = /(!\[.*?\]\()(.*?)(\))/g;
    }
    /**
     * Create a new RelativeIncludesConverterComponent instance.
     *
     * @param typedoc The application.
     */
    initialize(typedoc) {
        this._typedoc = typedoc;
        typedoc.converter.on(typedoc_1.Converter.EVENT_CREATE_DECLARATION, (c, r, n) => {
            const filePath = this.getNodeFilePath(n);
            if (!filePath)
                return;
            this._reflectionPaths.set(r, filePath);
        });
        typedoc.renderer.on(typedoc_1.Renderer.EVENT_BEGIN, () => {
            this._outFolder = this._typedoc?.options.getValue('out');
            if (!this._outFolder)
                return;
            this._outMediaFolder = path.join(this._outFolder, 'assets', 'media');
            if (!fs.existsSync(this._outMediaFolder)) {
                fs.mkdirSync(this._outMediaFolder, { recursive: true });
            }
        });
        let currentReflection = undefined;
        let currentOutputFilePath = undefined;
        typedoc.renderer.on(typedoc_1.PageEvent.BEGIN, (event) => {
            currentOutputFilePath = event.url;
            currentReflection =
                event.model instanceof typedoc_1.Reflection ? event.model : undefined;
        });
        typedoc.renderer.on(typedoc_1.MarkdownEvent.PARSE, (event) => {
            if (!currentOutputFilePath)
                return;
            if (!currentReflection)
                return;
            if (!this._outMediaFolder)
                return;
            const filePath = this._reflectionPaths.get(currentReflection);
            if (!filePath)
                return;
            event.parsedText = this.processText(event.parsedText, filePath, currentOutputFilePath);
        }, undefined, 1 // Do it before the default
        );
    }
    /**
     * Get the file the node was created from.
     *
     * @param node The node.
     * @returns The file path.
     */
    getNodeFilePath(node) {
        if (!node)
            return undefined;
        if ('fileName' in node)
            return node['fileName'];
        return this.getNodeFilePath(node.parent);
    }
    /**
     * Processes a text collecting the images from it and copying to the out folder.
     *
     * @param text The text to process.
     * @param originalFilePath Path of the parsed file.
     * @param outputFilePath Path of the documentation file.
     * @returns The modified comment.
     */
    processText(text, originalFilePath, outputFilePath) {
        return text.replace(this._imagePattern, (_match, prefix, pathGroup, suffix) => {
            if (typeof pathGroup === 'string' &&
                (pathGroup.startsWith('./') || pathGroup.startsWith('../'))) {
                const imagePath = path.join(path.dirname(originalFilePath), pathGroup);
                let newPath = this._imagePathMaps.get(imagePath);
                if (!newPath) {
                    const ext = path.extname(imagePath);
                    const fileName = path.basename(imagePath, ext);
                    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                    newPath = path.join(this._outMediaFolder, fileName + ext);
                    let index = 0;
                    while (this._usedOutputPaths.has(newPath)) {
                        newPath = path.join(
                        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                        this._outMediaFolder, `${fileName}_${index++}${ext}`);
                    }
                    this._usedOutputPaths.add(newPath);
                    this._imagePathMaps.set(imagePath, newPath);
                    fs.copyFileSync(imagePath, newPath);
                }
                return (prefix +
                    path
                        .relative(
                    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                    path.dirname(path.join(this._outFolder, outputFilePath)), 
                    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                    newPath)
                        .replace(/\\/g, '/') +
                    suffix);
            }
            else {
                return _match;
            }
        });
    }
}
exports.CopyAssets = CopyAssets;
//# sourceMappingURL=copy-assets.js.map